@@ -6,7 +6,8 @@ class C2PACK
     @itj,@tj,@iti,@ti = ISPACK::c2init(@jm,@im)
   end
 
-  attr_accessor :lm, :km, :jm, :im, :r
+#  attr_accessor :lm, :km, :jm, :im, :r
+  attr_reader :lm, :km, :jm, :im, :r, :itj, :tj, :iti, :ti
 
   def _alloc_w(length)
     if !@w || @w.length < length
@@ -75,7 +76,7 @@ class C2PACK
   def swck(avt, div, phi)
     _alloc_ws(((2*@km+1)*(@lm+1)))
     _alloc_wg(((@jm+1)*@im*4))
-    ISPACK::c2swck(@lm,@km,@jm,@r,@im,avt,div,phi,@ws,@wg,@itj,@tj,@iti,@ti)
+    ISPACK::c2swck(@lm,@km,@jm,@im,@r,avt,div,phi,@ws,@wg,@itj,@tj,@iti,@ti)
   end
   def swnl(avt, div, phi)
     _alloc_ws(((2*@km+1)*(@lm+1)))
@@ -97,7 +98,8 @@ class N2PACK
     @itj,@tj,@iti,@ti = ISPACK::n2init(@jm,@im)
   end
 
-  attr_accessor :lm, :km, :jm, :im
+#  attr_accessor :lm, :km, :jm, :im
+  attr_reader :lm, :km, :jm, :im, :itj, :tj, :iti, :ti
 
   def _alloc_w(length)
     if !@w || @w.length < length
@@ -159,7 +161,8 @@ class P2PACK
     @itj,@tj,@iti,@ti = ISPACK::p2init(@jm,@im)
   end
 
-  attr_accessor :lm, :km, :jm, :im, :r
+#  attr_accessor :lm, :km, :jm, :im, :r
+  attr_reader :lm, :km, :jm, :im, :r, :itj, :tj, :iti, :ti
 
   def _alloc_w(length)
     if !@w || @w.length < length
@@ -226,7 +229,8 @@ class P3PACK
     @itk,@tk,@itj,@tj,@iti,@ti = ISPACK::p3init(@jm,@im)
   end
 
-  attr_accessor :lm, :km, :jm, :im
+#  attr_accessor :lm, :km, :jm, :im
+  attr_reader :nm, :mm, :lm, :km, :jm, :im, :itk, :tk, :itj, :tj, :iti, :ti
 
   def _alloc_w(length)
     if !@w || @w.length < length
@@ -273,7 +277,9 @@ class SMPACK
     @x,@y,@w,@it,@t,@ip,@p,@qsg,@qgs,@qsv,@qvs,@qsu,@qus,@r,@ml,@nl,@rf,@rb = ISPACK::sminit(@mm,@im,@jm,@km)
   end
 
-  attr_accessor :mm, :im, :jm, :km
+#  attr_accessor :mm, :im, :jm, :km
+  attr_reader :mm, :im, :jm, :km, :x, :y, :w, :it, :t, :ip, :p, 
+    :qsg, :qgs, :qsv, :qvs, :qsu, :qus, :r, :ml, :nl, :rf, :rb
 
   def _alloc_work(length)
     if !@work || @work.length < length
@@ -297,31 +303,34 @@ class SMPACK
     _alloc_work(id*jd*@km*2)
     ISPACK::smtg2s(@mm,@im,id,@jm,jd,@km,g,@work,@it,@t,@ip,@p,@qgs).reshape!((@mm+1)*(@mm+1),@km)
   end
-  def ts1v(a, id=@im/2*2+1, jd=@jm/2*2+1)
+#  def ts1v(a, id=@im/2*2+1, jd=@jm/2*2+1)
+  def ts1v(a, id=@im, jd=@jm)
     _alloc_work(id*jd*@km*2)
-    u,v = ISPACK::smts1v(@mm,@im,id,@jm,jd,@km,a,@work,@it,@t,@ip,@p,@qsv)
+    u,v = ISPACK::smts1v(@mm,@im,id,@jm,jd,@km,a,@work,@it,@t,@ip,@p,@qsv,@r,@ml)
     return u.reshape!(id,jd,@km), v.reshape!(id,jd,@km)
   end
-  def ts2g(s, id=@im/2*2+1, jd=@jm/2*2+1)
+#  def ts2g(s, id=@im/2*2+1, jd=@jm/2*2+1)
+  def ts2g(s, id=@im, jd=@jm)
     _alloc_work(id*jd*@km)
     ISPACK::smts2g(@mm,@im,id,@jm,jd,@km,s,@work,@it,@t,@ip,@p,@qsg).reshape!(@id,@jd,@km)
   end
-  def ts2v(a, b, id=@im/2*2+1, jd=@jm/2*2+1)
+#  def ts2v(a, b, id=@im/2*2+1, jd=@jm/2*2+1)
+  def ts2v(a, b, id=@im, jd=@jm)
     _alloc_work(id*jd*@km*2)
-    u,v = ISPACK::smts2v(@mm,@im,id,@jm,jd,@km,a,b,@work,@it,@t,@ip,@p,@qsv)
+    u,v = ISPACK::smts2v(@mm,@im,id,@jm,jd,@km,a,b,@work,@it,@t,@ip,@p,@qsv,@r,@ml)
     return u.reshape!(id,jd,@km), v.reshape!(id,jd,@km)
   end
   def tv1s(u, v)
     id = u.shape[0]
     jd = u.shape[1]
     _alloc_work(id*jd*@km*3)
-    ISPACK::smtv1s(@mm,@im,id,@jm,jd,@km,u,v,@work,@it,@t,@ip,@p,@qvs).reshape!(id,jd,@km).reshape!((@mm+1)*(@mm+1),@km)
+    ISPACK::smtv1s(@mm,@im,id,@jm,jd,@km,u,v,@work,@it,@t,@ip,@p,@qvs,@r,@ml).reshape!(id,jd,@km).reshape!((@mm+1)*(@mm+1),@km)
   end
   def tv2s(u, v)
     id = u.shape[0]
     jd = u.shape[1]
     _alloc_work(id*jd*@km*3)
-    a,b = ISPACK::smtv2s(@mm,@im,id,@jm,jd,@km,u,v,@work,@it,@t,@ip,@p,@qvs)
+    a,b = ISPACK::smtv2s(@mm,@im,id,@jm,jd,@km,u,v,@work,@it,@t,@ip,@p,@qvs,@r,@ml)
     return a.reshape!((@mm+1)*(@mm+1),@km), b.reshape!((@mm+1)*(@mm+1),@km)
   end
 end
@@ -330,12 +339,16 @@ class SNPACK
   def initialize(mm,im,jm,km)
     @mm,@im,@jm,@km = mm,im,jm,km
     @it,@t,@y,@ip,@p,@r,@ia,@a = ISPACK::sninit(@mm,@im,@jm)
-    if km > 1
+    if @km > 1
       @ipk,@pk,@rk = ISPACK::snkini(@mm,@jm,@km,@ip,@p,@r)
     end
   end
 
-  attr_accessor :mm, :im, :jm, :km
+#  attr_accessor :mm, :im, :jm, :km
+  attr_reader :mm, :im, :jm, :km, :it, :t, :y, :ip, :p, :r, :ia, :a
+#  if @km > 1
+#    attr_reader :ipk, :pk, :rk
+#  end
 
   def _alloc_q(length)
     if !@q || @q.length < length
@@ -368,23 +381,21 @@ class SNPACK
     _alloc_ws(size_w)
     _alloc_ww(size_w)
     if @km > 1
-      ISPACK::sntg2s(@mm,@im,id,@jm,jd,@km,g,@it,@t,@ipk,@pk,@rk,@ia,@a,@q,@ws,@ww,ipow,iflag).reshape!((@mm+1)*(@mm+1),@km)
+      ISPACK::sntg2s(@mm,@im,id,@jm,jd,@km,g,@it,@t,@y,@ipk,@pk,@rk,@ia,@a,@q,@ws,@ww,ipow,iflag).reshape!((@mm+1)*(@mm+1),@km)
     else
-#      ISPACK::sntg2s(@mm,@im,id,@jm,jd,@km,g,@it,@t,@ip,@p,@r,@ia,@a,@q,@ws,@ww,ipow,iflag).reshape!((@mm+1)*(@mm+1),@km)
-#      p @mm,@im,id,@jm,jd,@km,g[0],g[1],g[2],g[3],g[4],g[5],g[6],g[7],g[8],g[9]
-#      p @it[0],@it[1],@it[2],@it[3],@it[4]
       ISPACK::sntg2s(@mm,@im,id,@jm,jd,@km,g,@it,@t,@y,@ip,@p,@r,@ia,@a,@q,@ws,@ww,ipow,iflag)
     end
   end
-  def ts2g(s, ipow, iflag, id=@im/2*2+1, jd=@jm/2*2+1 )
+#  def ts2g(s, ipow, iflag, id=@im/2*2+1, jd=@jm/2*2+1 )
+  def ts2g(s, ipow, iflag, id=@im, jd=@jm )
     _alloc_q((@km*((@mm+1)/2+@mm+1)*@jm))
     size_w = @km*[((@mm+1)/2*2+3)*(@mm/2+2)*2,jd*((@mm+1)/2+@mm+1)*2,jd*@im].max
     _alloc_ws(size_w)
     _alloc_ww(size_w)
     if @km > 1
-      ISPACK::snts2g(@mm,@im,id,@jm,jd,@km,s,@it,@t,@ipk,@pk,@rk,@ia,@a,@q,@ws,@ww,ipow,iflag).reshape!(id,jd,@km)
+      ISPACK::snts2g(@mm,@im,id,@jm,jd,@km,s,@it,@t,@y,@ipk,@pk,@rk,@ia,@a,@q,@ws,@ww,ipow,iflag).reshape!(id,jd,@km)
     else
-      ISPACK::snts2g(@mm,@im,id,@jm,jd,@km,s,@it,@t,@ip,@p,@r,@ia,@a,@q,@ws,@ww,ipow,iflag).reshape!(id,jd)
+      ISPACK::snts2g(@mm,@im,id,@jm,jd,@km,s,@it,@t,@y,@ip,@p,@r,@ia,@a,@q,@ws,@ww,ipow,iflag).reshape!(id,jd)
     end
   end
 end
@@ -396,12 +407,13 @@ class SOPACK < SNPACK
     @ib,@b = ISPACK::soinit(mm)
   end
 
-  def ndnl(s, id=@im/2*2+1, jd=@jm/2*2+1)
+#  def ndnl(s, id=@im/2*2+1, jd=@jm/2*2+1)
+  def ndnl(s, id=@im, jd=@jm)
     _alloc_q((2*((@mm+2)/2+@mm+2)*@jm))
     size_w = @km*[((@mm+1)/2*2+3)*(@mm/2+2)*2,jd*((@mm+1)/2+@mm+1)*2,jd*id].max
     _alloc_ws(size_w)
     _alloc_ww(size_w)
-    ISPACK::sondnl(@mm,@im,id,@jm,jd,s,@it,@t,@ipk,@pk,@rk,@ia,@a,@ib,@b,@q,@ws,@ww)
+    ISPACK::sondnl(@mm,@im,id,@jm,jd,s,@it,@t,@y,@ipk,@pk,@rk,@ia,@a,@ib,@b,@q,@ws,@ww)
   end
 end
 
@@ -411,7 +423,8 @@ class STPACK
     @q,@r,@it,@t = ISPACK::stinit(@mm,@jm,@im)
   end
 
-  attr_accessor :mm, :jm, :im
+#  attr_accessor :mm, :jm, :im
+  attr_reader :mm, :jm, :im, :q, :r, :it, :t
 
   def _alloc_p(length)
     if !@p || @p.length < length
@@ -473,7 +486,8 @@ class LTPACK
     @q,@r = ISPACK::ltinit(@mm,@jm)
   end
 
-  attr_accessor :mm, :jm
+#  attr_accessor :mm, :jm
+  attr_reader :mm, :jm, :q, :r
 
   def _alloc_p(length)
     if !@p || @p.length < length
@@ -548,13 +562,23 @@ class LTPACK
 end
 
 class SPPACK < SNPACK
-  def initialize(mm,im,jm)
+  def initialize(mm,im,jm,id=nil,jd=nil)
     @mm,@im,@jm = mm,im,jm
     @it,@t,@y,@ip,@p,@r,@ia,@a = ISPACK::sninit(@mm,@im,@jm)
     @rn = ISPACK::spnini(@mm)
     @irm = ISPACK::spmini(@mm)
-    @id = @im/2*2+1 # least odd number >= @im
-    @jd = @jm/2*2+1 # least odd number >= @jm
+#    @id = @im/2*2+1 # least odd number >= @im
+#    @jd = @jm/2*2+1 # least odd number >= @jm
+    if(id)
+      @id = id
+    else
+      @id = @im
+    end
+    if(jd)
+      @jd = jd
+    else
+      @jd = @jm
+    end
     @km = 1
   end
 
@@ -641,12 +665,12 @@ class SPPACK < SNPACK
     _alloc_ww(size_w)
     ISPACK::spswhw(@mm,@im,@id,@jm,@jd,omega,barphi,dnu,alpha,lev,avt,div,phi,@rn,@irm,@it,@t,@y,@ip,@p,@r,@ia,@a,@q,@ws,@ww)
   end
-  def swhx(omega, barphi, dnu, alpha, lev, avt, div, phi)
+  def swhx(omega, barphi, dnu, lev, avt, div, phi)
     _alloc_q((((@mm+1)/2+@mm+1)*@jm))
     size_w = 10*[((@mm+1)/2*2+3)*(@mm/2+2)*2,@jd*((@mm+1)/2+@mm+1)*2,@jd*@id].max
     _alloc_ws(size_w)
     _alloc_ww(size_w)
-    ISPACK::spswhx(@mm,@im,@id,@jm,@jd,omega,barphi,dnu,alpha,lev,avt,div,phi,@rn,@irm,@it,@t,@y,@ip,@p,@r,@ia,@a,@q,@ws,@ww)
+    ISPACK::spswhx(@mm,@im,@id,@jm,@jd,omega,barphi,dnu,lev,avt,div,phi,@rn,@irm,@it,@t,@y,@ip,@p,@r,@ia,@a,@q,@ws,@ww)
   end
   def swli(barphi, dnu, alpha, dt)
     ISPACK::spswli(@mm,barphi,dnu,alpha,dt)
@@ -678,12 +702,12 @@ class SPPACK < SNPACK
     _alloc_ww(size_w)
     ISPACK::spswnw(@mm,@im,@id,@jm,@jd,omega,barphi,dnu,alpha,avt,div,phi,@rn,@irm,@it,@t,@y,@ip,@p,@r,@ia,@a,@q,@ws,@ww)
   end
-  def swnx(omega, barphi, dnu, alpha, avt, div, phi)
+  def swnx(omega, barphi, dnu, avt, div, phi)
     _alloc_q((((@mm+1)/2+@mm+1)*@jm))
     size_w = 7*[((@mm+1)/2*2+3)*(@mm/2+2)*2,@jd*((@mm+1)/2+@mm+1)*2,@jd*@id].max
     _alloc_ws(size_w)
     _alloc_ww(size_w)
-    ISPACK::spswnx(@mm,@im,@id,@jm,@jd,omega,barphi,dnu,alpha,avt,div,phi,@rn,@irm,@it,@t,@y,@ip,@p,@r,@ia,@a,@q,@ws,@ww)
+    ISPACK::spswnx(@mm,@im,@id,@jm,@jd,omega,barphi,dnu,avt,div,phi,@rn,@irm,@it,@t,@y,@ip,@p,@r,@ia,@a,@q,@ws,@ww)
   end
 end
 
@@ -692,7 +716,8 @@ class FTTZL
     @n = n
     @it,@t = ISPACK::fttzli(n)
   end
-  attr_accessor :n
+#  attr_accessor :n
+  attr_reader :n, :it, :t
 
   def _alloc_y(length)
     if !@y || @y.length < length
@@ -715,7 +740,8 @@ class FTTZU
     @n = n
     @it,@t = ISPACK::fttzui(n)
   end
-  attr_accessor :n
+#  attr_accessor :n
+  attr_reader :n, :it, :t
 
   def _alloc_y(length)
     if !@y || @y.length < length
@@ -745,7 +771,8 @@ class FTTRU
     @n = n
     @it,@t = ISPACK::fttrui(n)
   end
-  attr_accessor :n
+#  attr_accessor :n
+  attr_reader :n, :it, :t
 
   def _alloc_y(length)
     if !@y || @y.length < length
@@ -775,7 +802,8 @@ class FTTCT
     @n = n
     @it,@t = ISPACK::fttcti(n)
   end
-  attr_accessor :n
+#  attr_accessor :n
+  attr_reader :n, :it, :t
 
   def _alloc_y(length)
     if !@y || @y.length < length
@@ -805,7 +833,8 @@ class FTTST
     @n = n
     @it,@t = ISPACK::fttsti(n)
   end
-  attr_accessor :n
+#  attr_accessor :n
+  attr_reader :n, :it, :t
 
   def _alloc_y(length)
     if !@y || @y.length < length
@@ -835,7 +864,8 @@ class FTTCM
     @n = n
     @it,@t = ISPACK::fttcmi(n)
   end
-  attr_accessor :n
+#  attr_accessor :n
+  attr_reader :n, :it, :t
 
   def _alloc_y(length)
     if !@y || @y.length < length
@@ -865,7 +895,8 @@ class FTTSM
     @n = n
     @it,@t = ISPACK::fttsmi(n)
   end
-  attr_accessor :n
+#  attr_accessor :n
+  attr_reader :n, :it, :t
 
   def _alloc_y(length)
     if !@y || @y.length < length
@@ -891,7 +922,8 @@ class FTTSM
 end
 
 
-class NDPACK
+#class NDPACK
+module ND
   def ca2p(omg, avt)
     @mm = (avt.length).sqrt-1
     ISPACK::ndca2p(@mm,omg,avt)
