@@ -17,7 +17,9 @@
 # 05/06/24   ver.12 ̥åѡ
 # 05/07/20   ver.13 Ĺåshapetotalѹ1(ver11)ϼ
 # 05/08/05   ver.14 ispack-0.71򻲾Ȥ褦ѹ
-# 05/10/03   ver.15 P2PACK,SPPACKθб
+# 05/10/03   ver.15 P2PACK,SPPACK,SOPACKθб
+# 05/10/12   ver.16 SMPACK,LTPACKΥХ
+#                   CåѡκˡѹC2SWCK,SPSWNX,SPSWHXΥХfix
 #-----------------------------------------------------------------------------#
 
 libfilename = "ispack_raw" # .so ˻Ȥ̾
@@ -39,7 +41,8 @@ filenames = [
   srcpath+"/smpack/doc/smpack.tex",
 #  srcpath+"/snpack/doc/snpack.tex", # 
   "./snpack.tex", 
-  srcpath+"/sopack/doc/sopack.tex",
+#  srcpath+"/sopack/doc/sopack.tex", # 
+  "./sopack.tex", 
 #  srcpath+"/c2pack/doc/c2pack.tex", # ̤ĤƤʤΤ롢(6/17)
   "./c2pack.tex",
   srcpath+"/ftpack/doc/ftpack.tex",  # ޥ˥奢뤬ü
@@ -64,6 +67,7 @@ filenames = [ # for debug
 
 routinelist = Array.new
 varlist = Hash.new
+vars1 = Hash.new
 vars = Hash.new
 
 filenames.each{ |filename|
@@ -123,12 +127,12 @@ loop do
 
     if ( !tail1 ) # for C2PACK
       line = line.chomp + file.gets # 
-      p line
+#      p line
       if line =~ /^.*\{\\tt #{@subroutine}\((.*)\)\}(\\\\)*$/ # for ST,LT,FT,C2,SP
         list = $1
 #        @varlist = list.split(",")
         @varlist = list.split(/[,| ]/)
-        p @varlist
+#        p @varlist
       else
         raise "invalid lines? "+line
       end
@@ -139,7 +143,8 @@ loop do
 #    @varlist = $1.split(",")
 #    p @varlist
     varlist[@subroutine] = @varlist
-    vars[@subroutine] = Array.new
+    vars1[@subroutine] = Array.new
+    vars[@subroutine] = Hash.new
 
 #    if sub1 # ιԤɤ
     if tail3 # ιԤɤ
@@ -150,7 +155,8 @@ loop do
         @varlist = $1.split(",")
         varlist[@subroutine1] = @varlist
 #        p varlist[@subroutine1]
-        vars[@subroutine1] = Array.new
+        vars1[@subroutine1] = Array.new
+        vars[@subroutine1] = Hash.new
 
         if sub2 # ιԤɤ
 #          p @subroutine2
@@ -159,13 +165,19 @@ loop do
             @varlist = $1.split(",")
             varlist[@subroutine2] = @varlist
 #            p varlist[@subroutine2]
-            vars[@subroutine2] = Array.new
+            vars1[@subroutine2] = Array.new
+            vars[@subroutine2] = Hash.new
           end
         end
       end
     end
   end
 
+#  if line =~ /^\{\\tt (.*)\} *& *\{\\tt (.*)\} *& *$/
+#      line = line.chomp + file.gets
+#      p line
+#  end
+
   if line =~ /^\{\\tt (.*)\} +& +\{\\tt (.*)\}/
     unless line =~ /^\{\\tt (.*)\} +& +\{\\tt (.*)\}.*\\\\.*$/ # ³
 #      p line
@@ -176,7 +188,8 @@ loop do
 
 
 #  if line =~ /^\{\\tt (.*)\} +& +\{\\tt (.*)\} +& +([к])/
-  if line =~ /^ *\{\\tt +([^ ]+) *\} *& *\{\\tt (.*)\} *& *([к])/ # for FTPACK
+#  if line =~ /^ *\{\\tt +([^ ]+) *\} *& *\{\\tt (.*)\} *& *([к])/ # for FTPACK
+  if line =~ /^ *\{\\tt +([^ ]+) *\} *& *\{\\tt (.*)\} *& *\t*([к])/ # for FT,SMPACK
     @name = $1
     type = $2
     case($3)
@@ -204,11 +217,11 @@ loop do
     if line =~ /^ *\{\\tt .*\} *& *\{\\tt .*\} *&.*礭(.+)/ # for FTPACK
       type1 = $1
       if type == "(D)" || type == "(I)"
-        p type
+#        p type
         type = type.gsub(')','(')
         type += type1.gsub(/^\\tt/,"").gsub("$\\times$","*").gsub("\\tt","*").gsub("**","*")
         type += "))"
-        p type
+#        p type
       else
         raise "unsupported variable type"
       end
@@ -224,7 +237,8 @@ loop do
 #      print "#{@name} is not listed in the arguments of #{@subroutine}\n" ## for debug
     else
       var = {"name"=>@name.downcase, "type"=>type, "inout"=>inout}
-      vars[@subroutine].push(var)
+      vars1[@subroutine].push(var)
+      vars[@subroutine][@name.downcase] = {"type"=>type, "inout"=>inout}
     end
 
     if @subroutine1
@@ -232,7 +246,8 @@ loop do
 #        print "#{@name} is not listed in the arguments of #{@subroutine1}\n" ## for debug
       else
         var = {"name"=>@name.downcase, "type"=>type, "inout"=>inout}
-        vars[@subroutine1].push(var)
+        vars1[@subroutine1].push(var)
+        vars[@subroutine1][@name.downcase] = {"type"=>type, "inout"=>inout}
       end
     end
 
@@ -241,96 +256,131 @@ loop do
 #        print "#{@name} is not listed in the arguments of #{@subroutine2}\n" ## for debug
       else
         var = {"name"=>@name.downcase, "type"=>type, "inout"=>inout}
-        vars[@subroutine2].push(var)
+        vars1[@subroutine2].push(var)
+        vars[@subroutine2][@name.downcase] = {"type"=>type, "inout"=>inout}
       end
     end
 
 #    var = {"name"=>name.downcase, "type"=>type, "inout"=>inout}
-#    vars[@subroutine].push(var)
+#    vars1[@subroutine].push(var)
 
   end
 
   if line =~ /^ *& +& +([к])/ # ξѿ
 #    p line
     inout1 = $1
-    if vars[@subroutine][-1]["name"] == @name.downcase
-      inout = vars[@subroutine][-1]["inout"]
+    if vars1[@subroutine][-1]["name"] == @name.downcase
+      inout = vars1[@subroutine][-1]["inout"]
 #      if (inout == "in" && $1 =~ //) || (inout == "out" && $1 == //)
       if (inout == "in" && inout1 =~ //) || (inout == "out" && inout1 == //)
-        vars[@subroutine][-1]["inout"] = "inout"
+        vars1[@subroutine][-1]["inout"] = "inout"
       else
         raise "unsupported inout type"
       end
 #    else
-#      p vars[@subroutine][-1]["name"]
+#      p vars1[@subroutine][-1]["name"]
 #      p @name.downcase
     end
-    if @subroutine1 && vars[@subroutine1][-1]["name"] == @name.downcase
-      inout = vars[@subroutine1][-1]["inout"]
+    if @subroutine1 && vars1[@subroutine1][-1]["name"] == @name.downcase
+      inout = vars1[@subroutine1][-1]["inout"]
 #      if (inout == "in" && $1 =~ //) || (inout == "out" && $1 == //)
       if (inout == "in" && inout1 =~ //) || (inout == "out" && inout1 == //)
-        vars[@subroutine1][-1]["inout"] = "inout"
+        vars1[@subroutine1][-1]["inout"] = "inout"
       else
         raise "unsupported inout type"
       end
     end
-    if @subroutine2 && vars[@subroutine2][-1]["name"] == @name.downcase
-      inout = vars[@subroutine2][-1]["inout"]
+    if @subroutine2 && vars1[@subroutine2][-1]["name"] == @name.downcase
+      inout = vars1[@subroutine2][-1]["inout"]
 #      if (inout == "in" && $1 =~ //) || (inout == "out" && $1 == //)
       if (inout == "in" && inout1 =~ //) || (inout == "out" && inout1 == //)
-        vars[@subroutine2][-1]["inout"] = "inout"
+        vars1[@subroutine2][-1]["inout"] = "inout"
       else
         p inout
         raise "unsupported inout type"
       end
     end
+
+    if vars[@subroutine][@name.downcase]
+      inout = vars[@subroutine][@name.downcase]["inout"]
+      if (inout == "in" && inout1 =~ //) || (inout == "out" && inout1 == //)
+        vars[@subroutine][@name.downcase]["inout"] = "inout"
+      else
+        raise "unsupported inout type"
+      end
+    end
+
+    if @subroutine1
+      inout = vars[@subroutine1][@name.downcase]["inout"]
+      if (inout == "in" && inout1 =~ //) || (inout == "out" && inout1 == //)
+        vars[@subroutine1][@name.downcase]["inout"] = "inout"
+      else
+        raise "unsupported inout type"
+      end
+    end
+
+    if @subroutine2
+      inout = vars[@subroutine2][@name.downcase]["inout"]
+      if (inout == "in" && inout1 =~ //) || (inout == "out" && inout1 == //)
+        vars[@subroutine2][@name.downcase]["inout"] = "inout"
+      else
+        raise "unsupported inout type"
+      end
+    end
+
   end
 
 end
 
 #p routinelist
 #p varlist
-#p vars
+#p vars1
+#routinelist.each{ |subroutine| 
+#  p subroutine
+#  p varlist[subroutine].length
+#  p vars1[subroutine].length
+#}
 
 file.close
 
 } # end of filenames
 
 
-
 # 㳰
+
+=begin
+vars1["SPSWNW"] = vars1["SPSWNV"] if vars1["SPSWNV"]
+vars1["SPSWNX"] = vars1["SPSWNV"] if vars1["SPSWNV"]
+vars1["SPSWHW"] = vars1["SPSWHV"] if vars1["SPSWHV"]
+vars1["SPSWHX"] = vars1["SPSWHV"] if vars1["SPSWHV"]
+=end
+
 vars["SPSWNW"] = vars["SPSWNV"] if vars["SPSWNV"]
 vars["SPSWNX"] = vars["SPSWNV"] if vars["SPSWNV"]
 vars["SPSWHW"] = vars["SPSWHV"] if vars["SPSWHV"]
 vars["SPSWHX"] = vars["SPSWHV"] if vars["SPSWHV"]
 
-#p vars["LTS2GZ"]
+#p vars1["LTS2GZ"]
 ["LTS2GZ","LTS2GW","LTS2VZ","LTS2VW","LTSRVZ","LTSRVW"].each{ |subroutine|
-  vars[subroutine].push({"name"=>"im", "type"=>"(I)", "inout"=>"in"}) if vars[subroutine]
+#  vars1[subroutine].push({"name"=>"im", "type"=>"(I)", "inout"=>"in"}) if vars1[subroutine]
+#  vars[subroutine]["im"] = {"type"=>"(I)", "inout"=>"in"} if vars[subroutine]
+#  varlist[subroutine].push("im") if vars[subroutine]
 }
 
-vars["C2S2GA"] = vars["C2S2GA"][0..3] + 
+=begin
+vars1["C2S2GA"] = vars1["C2S2GA"][0..3] + 
   [{"name"=>"s", "type"=>"(D((2*KM+1)*LM))", "inout"=>"in"}] + 
-  vars["C2S2GA"][4..-1] if vars["C2S2GA"]
-vars["C2G2SA"] = vars["C2G2SA"][0..4] + 
+  vars1["C2S2GA"][4..-1] if vars1["C2S2GA"]
+vars1["C2G2SA"] = vars1["C2G2SA"][0..4] + 
   [{"name"=>"s", "type"=>"(D((2*KM+1)*(LM+1)))", "inout"=>"out"}] + 
-  vars["C2G2SA"][5..-1] if vars["C2G2SA"]
+  vars1["C2G2SA"][5..-1] if vars1["C2G2SA"]
+=end
+
+vars["C2S2GA"]["s"] = {"type"=>"(D((2*KM+1)*LM))", "inout"=>"in"} if vars1["C2S2GA"]
+vars["C2G2SA"]["s"] = {"type"=>"(D((2*KM+1)*(LM+1)))", "inout"=>"out"} if vars1["C2G2SA"]
 
 # 㳰ޤ
 
-=begin
-# ưɲ
-routinelist.push("SONDNL2")
-varlist["SONDNL2"] = 
-  varlist["SONDNL"][0..6] + ["itp2ab","tp2r2ab"] + varlist["SONDNL"][16..-1]
-vars["SONDNL2"] = 
-  vars["SONDNL"][0..6] + 
-  [{"name"=>"itp2ab", "type"=>"(I(#{vars["SONDNL"][7]["type"][2..-1]}+#{vars["SONDNL"][9]["type"][2..-1]}+#{vars["SONDNL"][12]["type"][2..-1]}+#{vars["SONDNL"][14]["type"][2..-1]}))", "inout"=>"in" }] + 
-  [{"name"=>"tp2r2ab", "type"=>"(D(#{vars["SONDNL"][8]["type"][2..-1]}+#{vars["SONDNL"][10]["type"][2..-1]}+#{vars["SONDNL"][11]["type"][2..-1]}+#{vars["SONDNL"][13]["type"][2..-1]}+#{vars["SONDNL"][15]["type"][2..-1]}))", "inout"=>"in" }] + 
-  vars["SONDNL"][16..-1]
-
-# ưɲäޤ
-=end
 
 #-----------------------------------------------------------------------------#
 if(cwrapper) # Cåѡ񤭽Ф
@@ -350,16 +400,18 @@ routinelist.each{ |subroutine|
     raise "varlist for #{subroutine} not exist"
   end
 #  p varlist[subroutine].length
-#  p vars[subroutine].length
+#  p vars1[subroutine].length
 #  p varlist[subroutine]
-#  p vars[subroutine].each{|v| p v["name"]}
+#  p vars1[subroutine].each{|v| p v["name"]}
 
 #  varlist[subroutine].each{ |varname|
 #    p varname
-#    p vars[varname]
-#    if vars[varname]
+#    p vars1[varname]
+#    if vars1[varname]
 #  }
 
+=begin
+
 #  inlist = "self"
   invars = Array.new
   outvars = Array.new
@@ -369,9 +421,10 @@ routinelist.each{ |subroutine|
   c2r = "\n"
   args = ""
 
-#p vars[subroutine]
 
-  vars[subroutine].each{|v|
+#p vars1[subroutine]
+
+  vars1[subroutine].each{|v|
 #  p v["inout"]
 #  if v["inout"]=="in" || v["inout"]=="inout"
     vname = v["name"]
@@ -540,7 +593,7 @@ routinelist.each{ |subroutine|
         else
           raise "error in determing arytype"
         end
-      p arylength
+#      p arylength
         declare += "    struct NARRAY *#{vname}1;\n"
         declare += "    VALUE #{vname};\n"
         declare_c2r = "    int rank=1,*shape;\n    shape = ALLOCA_N(int, rank);\n"
@@ -555,19 +608,192 @@ routinelist.each{ |subroutine|
 #      raise "#{v["type"]} not supported"
       end
 
-=begin
-    elsif v["inout"]=="work"
-#    p v
-      arylength = v["type"].downcase.sub(/\(d/,"").sub(/\)$/,"").gsub(/([a-z][a-z|0-9]*)/,'\1'+"1").gsub(/,/,"*")
-      arylength = "1" if arylength =~ /礭/ # ---------------------------
-      declare += "    double *#{vname}1;\n"
-      r2c += "    #{vname}1 = ALLOCA_N(double,"+arylength+");\n"
-      args += v["name"]+"1,"
+    end
+  } # end of vars1[subroutine].each
+
 =end
 
-    end
-  } # end of vars[subroutine].each
+  invars = Array.new
+  outvars = Array.new
+  declare = ""
+  declare_c2r = nil
+  r2c = "\n"
+  c2r = "\n"
+  args = ""
 
+  varlist[subroutine].each{|v|
+    vname = v.downcase
+    if vname =~ /hspace/
+      # do nothing
+    elsif !vars[subroutine][vname]
+      p vars[subroutine]
+      p vars1[subroutine]
+      raise "vars[#{subroutine}][#{vname}] not exist"
+    else
+#      p vname
+      inout = vars[subroutine][vname]["inout"]
+      type = vars[subroutine][vname]["type"]
+      if inout =="in" || inout =="inout" || inout =="work"
+        invars.push(vname)
+        outvars.push(vname) if inout =="inout"
+        case(type)
+        when("(I)")
+          declare += "    int #{vname}1;\n"
+          r2c += "    #{vname}1 = NUM2INT(#{vname});\n"
+          args += "&#{vname}1,"
+        when("(D)")
+          declare += "    double #{vname}1;\n"
+          r2c += "    #{vname}1 = NUM2DBL(#{vname});\n"
+          args += "&#{vname}1,"
+        when(/\(I\(/)
+          arylength = type.downcase.sub(/\(i/,"").sub(/\)$/,"").gsub(/([a-z][a-z|0-9]*)/,'\1'+"1").gsub(/,/,"*")
+          declare += "    struct NARRAY *#{vname}1;\n"
+          r2c += "    #{vname} = na_cast_object(#{vname}, NA_LINT);\n"
+          r2c += "    GetNArray(#{vname},#{vname}1);\n"
+          if inout =="work"
+            r2c += "    if ( #{vname}1->total < #{arylength} ) rb_raise(rb_eArgError, \"invalid ary size: #{vname} : %d < #{arylength} = %d \", #{vname}1->total, #{arylength} );\n"
+          else
+            r2c += "    if ( #{vname}1->total != #{arylength} ) rb_raise(rb_eArgError, \"invalid ary size: #{vname} : %d != #{arylength} = %d \", #{vname}1->total, #{arylength} );\n"
+          end
+          args += vname+"1->ptr,"
+        when(/\(D\(/)
+          arylength = type.downcase.sub(/\(d/,"").sub(/\)$/,"").gsub(/([a-z][a-z|0-9]*)/,'\1'+"1").gsub(/,/,"*")
+          declare += "    struct NARRAY *#{vname}1;\n"
+          r2c += "    #{vname} = na_cast_object(#{vname}, NA_DFLOAT);\n"
+          r2c += "    GetNArray(#{vname},#{vname}1);\n"
+          args += vname+"1->ptr,"
+
+          # 㳰
+
+          if( subroutine =~ /TOGRD/ && vname == "q" )
+            arylength = "jm1"  # ˤ
+            r2c += "    if ( #{vname}1->total < #{arylength} ) rb_raise(rb_eArgError, \"invalid ary size: #{vname} : %d < #{arylength} = %d \", #{vname}1->total, #{arylength} );\n"
+          elsif( subroutine =~ /^LT/ && vname =~ /[uvgp]/ )
+            arylength = "jm1"  # ˤ
+            r2c += "    if ( #{vname}1->total < #{arylength} ) rb_raise(rb_eArgError, \"invalid ary size: #{vname} : %d < #{arylength} = %d \", #{vname}1->total, #{arylength} );\n"
+          elsif( (subroutine == "NDTV2A" || subroutine == "NDTA2V" ) && vname == "vrt" )
+            arylength = "2"  # ˤ
+            r2c += "    if ( #{vname}1->total < #{arylength} ) rb_raise(rb_eArgError, \"invalid ary size: #{vname} : %d < #{arylength} = %d \", #{vname}1->total, #{arylength} );\n"
+          elsif( (subroutine == "C2S2GA" ) && vname == "s" )
+            arylength = "(2*km1+1)*lm1"
+            r2c += "    if ( #{vname}1->total < #{arylength} ) rb_raise(rb_eArgError, \"invalid ary size: #{vname} : %d < #{arylength} = %d \", #{vname}1->total, #{arylength} );\n"
+          elsif( arylength =~ /礭ηͻ/ )
+            if( subroutine =~ /^SN/ )
+#            arylength = "km1*((mm1+1)/2*2+3)*(mm1/2+2)*2" # 1
+#            arylength = "km1*(jd1*((mm1+1)/2+mm1+1)*2)" # 2
+              arylength = "km1*(jd1*im1)" # 3
+            elsif( subroutine =~ /^SO/ )
+#            arylength = "2*((mm1+1)/2*2+3)*(mm1/2+2)*2" # 1
+#            arylength = "2*(jd1*((mm1+1)/2+mm1+1)*2)" # 2
+              arylength = "2*(jd1*im1)" # 3
+            elsif( subroutine == "SPNJCB" )
+#            arylength = "3*((mm1+1)/2*2+3)*(mm1/2+2)*2" # 1
+#            arylength = "3*(jd1*((mm1+1)/2+mm1+1)*2)" # 2
+              arylength = "3*(jd1*im1)" # 3
+            elsif( subroutine == "SPSWNL" )
+#            arylength = "5*((mm1+1)/2*2+3)*(mm1/2+2)*2" # 1
+#            arylength = "5*(jd1*((mm1+1)/2+mm1+1)*2)" # 2
+              arylength = "5*(jd1*im1)" # 3
+            elsif( subroutine == "SPSWCK" )
+#            arylength = "4*((mm1+1)/2*2+3)*(mm1/2+2)*2" # 1
+#            arylength = "4*(jd1*((mm1+1)/2+mm1+1)*2)" # 2
+              arylength = "4*(jd1*im1)" # 3
+            elsif( subroutine == "SPSWNV" && vname == "ww" )
+#            arylength = "((mm1+1)/2*2+3)*(mm1/2+2)*2" # 1
+#            arylength = "(jd1*((mm1+1)/2+mm1+1)*2)" # 2
+              arylength = "(jd1*im1)" # 3
+            elsif( subroutine == "SPSWNV" && vname == "ws" )
+#            arylength = "8*((mm1+1)/2*2+3)*(mm1/2+2)*2" # 1
+#            arylength = "8*(jd1*((mm1+1)/2+mm1+1)*2)" # 2
+              arylength = "8*(jd1*im1)" # 3
+            elsif( subroutine =~ /^SPSWC/ && vname == "ww" )
+#            arylength = "((mm1+1)/2*2+3)*(mm1/2+2)*2" # 1
+#            arylength = "(jd1*((mm1+1)/2+mm1+1)*2)" # 2
+              arylength = "(jd1*im1)" # 3
+            elsif( subroutine =~ /^SPSWC/ && vname == "ws" )
+#            arylength = "8*((mm1+1)/2*2+3)*(mm1/2+2)*2" # 1
+#            arylength = "8*(jd1*((mm1+1)/2+mm1+1)*2)" # 2
+              arylength = "4*(jd1*im1)" # 3
+            elsif( subroutine == "SPSWHV" && vname == "ww" )
+#            arylength = "((mm1+1)/2*2+3)*(mm1/2+2)*2" # 1
+#            arylength = "(jd1*((mm1+1)/2+mm1+1)*2)" # 2
+              arylength = "(jd1*im1)" # 3
+            elsif( subroutine == "SPSWHV" && vname == "ws" )
+#            arylength = "8*((mm1+1)/2*2+3)*(mm1/2+2)*2" # 1
+#            arylength = "8*(jd1*((mm1+1)/2+mm1+1)*2)" # 2
+              arylength = "11*(jd1*im1)" # 3
+            elsif( subroutine == "SPSWNW" )
+              arylength = "8*(jd1*im1)" # 3
+            elsif( subroutine == "SPSWNX" )
+              arylength = "7*(jd1*im1)" # 3
+            elsif( subroutine == "SPSWHW" )
+              arylength = "11*(jd1*im1)" # 3
+            elsif( subroutine == "SPSWHX" )
+              arylength = "10*(jd1*im1)" # 3
+            end
+            r2c += "    if ( #{vname}1->total < #{arylength} ) rb_raise(rb_eArgError, \"invalid ary size: #{vname} : %d < #{arylength} = %d \", #{vname}1->total, #{arylength} );\n"
+
+          # 㳰ޤ
+
+          elsif inout =="work"
+            r2c += "    if ( #{vname}1->total < #{arylength} ) rb_raise(rb_eArgError, \"invalid ary size: #{vname} : %d < #{arylength} = %d \", #{vname}1->total, #{arylength} );\n"
+          else
+            r2c += "    if ( #{vname}1->total != #{arylength} ) rb_raise(rb_eArgError, \"invalid ary size: #{vname} : %d != #{arylength} = %d \", #{vname}1->total, #{arylength} );\n"
+          end
+
+          r2c.gsub!("lmnm1","(2*nm1+1)*(2*mm1+1)*(2*lm1+1)") # for P3PACK
+        end
+      elsif inout =="out"
+        outvars.push(vname)
+
+        case(type)
+        when("(I)")
+          declare += "    int #{vname}1;\n"
+          declare += "    VALUE #{vname};\n"
+          args += "&#{vname}1,"
+          c2r += "    #{vname} = INT2NUM(#{vname}1);\n"
+        when("(D)")
+          declare += "    double #{vname}1;\n"
+          declare += "    VALUE #{vname};\n"
+          args += "&#{vname}1,"
+          c2r += "    #{vname} = rb_float_new(#{vname}1);\n"
+        when(/\([DI]\(/)
+          if type =~ /\(D\(/
+            arytype = "NA_DFLOAT"
+            arylength = type.downcase.sub(/\(d/,"").sub(/\)$/,"").gsub(/([a-z][a-z|0-9]*)/,'\1'+"1").gsub(/,/,"*")
+            args += vname+"1->ptr,"
+          elsif type =~ /\(I\(/
+            arytype = "NA_LINT"
+            arylength = type.downcase.sub(/\(i/,"").sub(/\)$/,"").gsub(/([a-z][a-z|0-9]*)/,'\1'+"1").gsub(/,/,"*")
+            args += vname+"1->ptr,"
+          else
+            raise "error in determing arytype"
+          end
+          declare += "    struct NARRAY *#{vname}1;\n"
+          declare += "    VALUE #{vname};\n"
+          declare_c2r = "    int rank=1,*shape;\n    shape = ALLOCA_N(int, rank);\n"
+          r2c += "    shape[0] = "+arylength+";\n"
+          r2c += "    #{vname} = na_make_object( "+arytype+", rank, shape, cNArray );\n"
+          r2c += "    GetNArray(#{vname},#{vname}1);\n"
+
+          r2c.gsub!("lmnm1","(2*nm1+1)*(2*mm1+1)*(2*lm1+1)") # for P3PACK
+
+        else
+
+        end
+
+      end
+
+    end
+  }
+
+  case(subroutine) # 㳰
+  when "LTS2GZ","LTS2GW","LTS2VZ","LTS2VW","LTSRVZ","LTSRVW"
+    vname = "im"
+    invars.push(vname)
+    declare += "    int #{vname}1;\n"
+    r2c = "    #{vname}1 = NUM2INT(#{vname});\n" + r2c
+  end
 
 
   case(outvars.length)
@@ -647,7 +873,7 @@ routinelist.sort.each{ |subroutine|
   outvars = Array.new
   walloc = ""
 
-  vars[subroutine].each{|v|
+  vars1[subroutine].each{|v|
 
     vname = v["name"]
     if v["inout"]=="in" || v["inout"]=="inout"
@@ -670,7 +896,7 @@ routinelist.sort.each{ |subroutine|
 
     end
 
-  } # end of vars[subroutine].each
+  } # end of vars1[subroutine].each
 
   if subroutine[2..5] == "INIT"
     file << "  def initialize(#{argvars.join(", ")})\n"
